/*
 * core.c: send a notification by SMS using Free mobile API
 *
 * Copyright (c) 2015 Arnaud Patard <arnaud.patard@rtp-net.org>
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 3 of the License, or (at your
 * option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
 *
 */

#define _GNU_SOURCE
#include <stdio.h>
#include <errno.h>
#include <stdlib.h>
#include <string.h>
#include <curl/curl.h>
#include <core.h>

static size_t write_callback(char *ptr, size_t size, size_t nmemb, void *userdata)
{
	return size * nmemb;
}

int send_msg(struct conf *opts, char *msg)
{
	CURL *curl;
	CURLcode res;
	char *buf, *msg_enc;
	long httpcode = 0;
	int ret;

	if (!opts || !opts->url || !opts->user || !opts->pass || !msg)
		return -EINVAL;

	curl_global_init(CURL_GLOBAL_ALL);
	curl = curl_easy_init();
	if (!curl)
		return -EPROTO;

	curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, &write_callback);

	if (opts->ignorecerterrors) {
		curl_easy_setopt(curl, CURLOPT_SSL_VERIFYPEER, 0L);
		curl_easy_setopt(curl, CURLOPT_SSL_VERIFYHOST, 0L);
	}

	msg_enc = curl_easy_escape(curl, msg, strlen(msg));
	if (!msg_enc)
		return -ENOMEM;

	ret = asprintf(&buf, "%s?user=%s&pass=%s&msg=%s", opts->url, opts->user, opts->pass, msg_enc);
	if (ret == -1)
		return -ENOMEM;

	curl_free(msg_enc);

	if (opts->verbose)
		curl_easy_setopt(curl, CURLOPT_VERBOSE, 1L);

	curl_easy_setopt(curl, CURLOPT_URL, buf);
	curl_easy_setopt(curl, CURLOPT_HTTPGET, 1L);

	res = curl_easy_perform(curl);
	if( res != CURLE_OK)
		return -(CURL_ERR_OFS+res);

	free(buf);

	curl_easy_getinfo(curl, CURLINFO_RESPONSE_CODE, &httpcode);

	curl_easy_cleanup(curl);

	curl_global_cleanup();
	return httpcode;
}
